/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2003      by Advanced Data Controls, Corp
 *  Copyright (C) 2004 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN  
 * 
 *  上記著作権者は，以下の (1)〜(4) の条件か，Free Software Foundation 
 *  によって公表されている GNU General Public License の Version 2 に記
 *  述されている条件を満たす場合に限り，本ソフトウェア（本ソフトウェア
 *  を改変したものを含む．以下同じ）を使用・複製・改変・再配布（以下，
 *  利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，その適用可能性も
 *  含めて，いかなる保証も行わない．また，本ソフトウェアの利用により直
 *  接的または間接的に生じたいかなる損害に関しても，その責任を負わない．
 * 
 *  @(#) $Id: integrator.c,v 1.4 2004/05/27 07:32:22 honda Exp $
 *  
 */

/*
 * CQ-FRK-NXP-ARM 内部機能用ドライバ
 */
#include <t_services.h>
#include <s_services.h>
#include <cqnxparm.h>

#define	BAUDRATE	230400

#define	DLVAL		8
#define	DIVADDVAL	2
#define	MULVAL		9
#define	FDRVAL		((MULVAL <<4) | DIVADDVAL)

#define	LPC_U0RBR	0xE000C000
#define	LPC_U0THR	0xE000C000
#define	LPC_U0IER	0xE000C004
#define	LPC_U0LCR	0xE000C00C
#define	LPC_U0LSR	0xE000C014
#define	LPC_U0DLL	0xE000C000
#define	LPC_U0DLM	0xE000C004
#define	LPC_U0FCR	0xE000C008
/*
 * シリアルI/Oポート初期化ブロック
 */
SIOPINIB siopinib_table[TNUM_SIOP] = {
    {	(VP)LPC_U0LCR, (VP)LPC_U0DLM, (VP)LPC_U0DLL,
	(VP)LPC_U0THR, (VP)LPC_U0RBR,
	(VP)LPC_U0IER, (VP)LPC_U0FCR,
	(VP)LPC_U0LSR,
	INTNO_U0,
	DLVAL, FDRVAL
    }
};


/*
 *  シリアルI/Oポート管理ブロックのエリア
 */
SIOPCB  siopcb_table[TNUM_SIOP];


/*
 *  シリアルI/OポートIDから管理ブロックを取り出すためのマクロ
 */
#define INDEX_SIOP(siopid)  ((UINT)((siopid) - 1))
#define get_siopcb(siopid)  (&(siopcb_table[INDEX_SIOP(siopid)]))

#define	RDR	(1<<0)
#define	THRE	(1<<5)

/*
 * 文字を受信したか?
 */ 
Inline BOOL
uart_getready(SIOPCB *siopcb)
{
    return(((sil_rew_mem((VP)(siopcb->siopinib->uart_flag)) & RDR)!=0));
}

/*
 * 文字を送信できるか?
 */
Inline BOOL
uart_putready(SIOPCB *siopcb)
{
    return((sil_rew_mem((VP)(siopcb->siopinib->uart_flag)) & THRE)!=0);
}

/*
 *  受信した文字の取り出し
 */
Inline UB
uart_getchar(SIOPCB *siopcb)
{
    return(sil_rew_mem((VP)(siopcb->siopinib->uart_rxdata)));
}

/*
 *  送信する文字の書き込み
 */
Inline void
uart_putchar(SIOPCB *siopcb, UB c)
{
    sil_wrw_mem((VP)(siopcb->siopinib->uart_txdata),c);
}

#define	RBR_IE	(1<<0)
#define	THRE_IE	(1<<1)
/*
 *  送信割込み許可
 */
Inline void
uart_enable_send(SIOPCB *siopcb)
{
    SIOPINIB *inib = siopcb->siopinib;
    UW reg;

    reg = sil_rew_mem(inib->uart_ier) | THRE_IE;
    sil_wrw_mem((VP)(inib->uart_ier), reg);
}

/*
 *  送信割込み禁止
 */
Inline void
uart_disable_send(SIOPCB *siopcb)
{
    SIOPINIB *inib = siopcb->siopinib;
    UW reg;

    reg = sil_rew_mem(inib->uart_ier) & ~THRE_IE;
    sil_wrw_mem((VP)(inib->uart_ier), reg);
}


/*
 *  受信割込み許可
 */
Inline void
uart_enable_rcv(SIOPCB *siopcb)
{
    UW reg;
    SIOPINIB *inib = siopcb->siopinib;

    reg = sil_rew_mem(inib->uart_ier) | RBR_IE;
    sil_wrw_mem((VP)(inib->uart_ier), reg);
}

/*
 *  受信割込み禁止
 */
Inline void
uart_disable_rcv(SIOPCB *siopcb)
{
    SIOPINIB *inib = siopcb->siopinib;
    UW reg;

    reg = sil_rew_mem(inib->uart_ier) & ~RBR_IE;
    sil_wrw_mem((VP)(inib->uart_ier), reg);
}

/*
 * カーネル起動時のログ出力用の初期化
 */
void
init_uart(void)
{
    UW fdiv;

    /* Disable Interrupt */
    U0IER = 0;
    U0LCR = 0x83; /* 8bits, 1Stop, No Parity */
    fdiv = DLVAL;
    U0DLM = fdiv / 256;
    U0DLL = fdiv % 256;
    U0FDR = (MULVAL <<4) | DIVADDVAL;
    U0LCR = 0x03;
    U0FCR = 0x07;	/* Enable and reset TX and RX FIFO */
}

/*
 *  オンチップのUARTからのポーリング出力
 */
void
uart_putc(char c)
{
    while ((U0LSR & THRE) == 0)
	;
    U0THR = c;
}

/*
 *  SIOドライバの初期化ルーチン
 *  1ポートしかないため，あまり意味はない
 */
void
uart_initialize()
{
    SIOPCB  *siopcb;
    UINT    i;

    /*
     *  シリアルI/Oポート管理ブロックの初期化
     */
    for (siopcb = siopcb_table, i = 0; i < TNUM_SIOP; siopcb++, i++) {
        siopcb->siopinib = &(siopinib_table[i]);
        siopcb->openflag = FALSE;
        siopcb->sendflag = FALSE;
    }
}

/*
 * オープンしているポートがあるか
 */
BOOL
uart_openflag(void)
{
    return(siopcb_table[0].openflag);
}

/*
 * シリアルI/Oポートのオープン
 */
SIOPCB *
uart_opn_por(ID siopid, VP_INT exinf)
{
    int i;
    UW reg;
    
    SIOPCB      *siopcb;
    SIOPINIB  *siopinib;

    siopcb = get_siopcb(siopid);
    siopinib = siopcb->siopinib;

    sil_dly_nse(500);

    /* Disable Interrupt */
    sil_wrw_mem((VP)siopinib->uart_ier, 0);

    /* If there is any character beeing sent, wait until it drains */
    while ((sil_rew_mem((VP)(siopcb->siopinib->uart_flag)) & THRE)==0)
	;

    sil_wrw_mem((VP)siopinib->uart_lcr, 0x83);
    sil_wrw_mem((VP)siopinib->uart_dlm, siopinib->divval / 256);
    sil_wrw_mem((VP)siopinib->uart_dll, siopinib->divval % 256);
    sil_wrw_mem((VP)siopinib->uart_lcr, 0x03);
    /* reset receiver and transmitter */
    sil_wrw_mem((VP)siopinib->uart_fcr, 0x07);

    /*
     * 割込み関連の設定
     */
    siopcb->exinf = exinf;
    siopcb->getready = siopcb->putready = FALSE;
    siopcb->openflag = TRUE;

    sil_wrw_mem((VP)siopinib->uart_ier, RBR_IE | THRE_IE);
    ena_int(siopinib->devid);

    return(siopcb);    
}

/*
 *  シリアルI/Oポートのクローズ
 */
void
uart_cls_por(SIOPCB *siopcb)
{
    sil_wrw_mem((VP)siopcb->siopinib->uart_ier, 0);
    siopcb->openflag = FALSE;
}

/*
 *  シリアルI/Oポートへの文字送信
 */
BOOL
uart_snd_chr(SIOPCB *siopcb, char c)
{
    if (uart_putready(siopcb)){
        uart_putchar(siopcb, c);
        return(TRUE);
    }
    return(FALSE);
}

/*
 *  シリアルI/Oポートからの文字受信
 */
INT
uart_rcv_chr(SIOPCB *siopcb)
{
    if (uart_getready(siopcb)) {
        return((INT)(UB) uart_getchar(siopcb));
    }
    return(-1);
}

/*
 *  シリアルI/Oポートからのコールバックの許可
 */
void
uart_ena_cbr(SIOPCB *siopcb, UINT cbrtn)
{
    switch (cbrtn) {
        case SIO_ERDY_SND:
            uart_enable_send(siopcb);
            break;
        case SIO_ERDY_RCV:
            uart_enable_rcv(siopcb);
            break;
    }
}

/*
 *  シリアルI/Oポートからのコールバックの禁止
 */
void
uart_dis_cbr(SIOPCB *siopcb, UINT cbrtn)
{
    switch (cbrtn) {
        case SIO_ERDY_SND:
            uart_disable_send(siopcb);
            break;
        case SIO_ERDY_RCV:
            uart_disable_rcv(siopcb);
            break;
    }
}

#define	IIR_RDA		0x02
#define	IIR_THRE	0x01
/*
 *  シリアルI/Oポートに対する割込み処理
 */
static void
uart_isr_siop(UW isr, SIOPCB *siopcb)
{
    if (isr == IIR_RDA) {
        uart_ierdy_rcv(siopcb->exinf);
    } else if (isr == IIR_THRE) {
        uart_ierdy_snd(siopcb->exinf);
    }
}

/*
 *  SIOの割込みサービスルーチン
 */

void
uart0_handler()
{
    UW imr, isr;
    int i;
    SIOPCB *siopcb;

    isr = U0IIR;
    isr >>= 1;
    isr &= 7;
    if (isr & (IIR_RDA|IIR_THRE))
	uart_isr_siop(isr, &(siopcb_table[0]));
}
