/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *                2003      by  Advanced Data Controls, Corp
 * 
 *  上記著作権者は，以下の (1)〜(4) の条件か，Free Software Foundation 
 *  によって公表されている GNU General Public License の Version 2 に記
 *  述されている条件を満たす場合に限り，本ソフトウェア（本ソフトウェア
 *  を改変したものを含む．以下同じ）を使用・複製・改変・再配布（以下，
 *  利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，その適用可能性も
 *  含めて，いかなる保証も行わない．また，本ソフトウェアの利用により直
 *  接的または間接的に生じたいかなる損害に関しても，その責任を負わない．
 * 
 *  @(#) $Id: hw_timer.h,v 1.4 2007/01/05 02:02:38 honda Exp $
 */

/*
 *  CPU依存タイマモジュール（NXP LPC2388用）
 *  PITを使用
 */

#ifndef _HW_TIMER_H_
#define _HW_TIMER_H_

#include <s_services.h>
#include <cqnxparm.h>

#ifdef USE_TIMER0
#define	TSYS_TCR	T0TCR
#define	TSYS_CTCR	T0CTCR
#define	TSYS_PR		T0PR
#define	TSYS_MCR	T0MCR
#define	TSYS_MR0	T0MR0
#define	TSYS_IR		T0IR
#define	TSYS_TC		T0TC
#else
#define	TSYS_TCR	T2TCR
#define	TSYS_CTCR	T2CTCR
#define	TSYS_PR		T2PR
#define	TSYS_MCR	T2MCR
#define	TSYS_MR0	T2MR0
#define	TSYS_IR		T2IR
#define	TSYS_TC		T2TC
#endif

/*
 *  タイマ割込みの割込み番号
 */
#ifdef USE_TIMER0
#define	INHNO_TIMER	INHNO_TIMER0
#else
#define	INHNO_TIMER	INHNO_TIMER2
#endif

#ifndef _MACRO_ONLY

/*
 *  タイマ値の内部表現の型
 */
typedef UW    CLOCK;

/*
 *  タイマ値の内部表現とミリ秒・μ秒単位との変換
 *  TIMER_CLOCK はターゲットボード毎に設定
 */
#define TO_CLOCK(nume, deno) (TIMER_CLOCK * (nume) / (deno))
#define TO_USEC(clock)       ((clock) * 1000 / TIMER_CLOCK)

/*
 *  設定できる最大のタイマ周期（単位は内部表現）
 */
#define MAX_CLOCK    ((CLOCK) 0xffffffff)
/*
 *  タイマの現在値を割込み発生前の値とみなすかの判断
 */
#define GET_TOLERANCE    100
#define BEFORE_IREQ(clock) \
        ((clock) >= TO_CLOCK(TIC_NUME, TIC_DENO) - GET_TOLERANCE)

/*
 *  タイマの起動処理
 *
 *  タイマを初期化し，周期的なタイマ割込み要求を発生させる．
 */
Inline void
hw_timer_initialize()
{
    /*
     *  タイマ関連の設定. On reset TIMER2 is disabled.
     */
    PCONP |= (1 << 22);		/* PCTIM2 */
    /* タイマ停止 */
    TSYS_TCR = 0;
    TSYS_CTCR = 0;		/* Timer mode */
    TSYS_PR = 0;		/* No prescale */
#define	MR0I	(1<<0)
#define	MR0R	(1<<1)
    TSYS_MCR = MR0R | MR0I;	/* Reset on MR0. Interrupt on MR0 */
    /* カウント値をセット */
    TSYS_MR0 = FPCLK/1000;
    /* カウントスタート   */
    TSYS_TCR = 1;
    /* 割込み許可 */        
    VICIntEnable = (1 << INHNO_TIMER);
}

/*
 *  タイマ割込み要求のクリア
 */
Inline void
hw_timer_int_clear()
{
    int ir;

    ir = TSYS_IR;
    TSYS_IR = ir;
}

/*
 *  タイマの停止処理
 */
Inline void
hw_timer_terminate()
{
    int ir;

    /* ペンディングビットをクリア */
    ir = TSYS_IR;
    TSYS_IR = ir;
    /* タイマ停止 */
    TSYS_TCR = 0;
    /* 割込み不許可*/
    VICIntEnClr = (1 << INHNO_TIMER);
}

/*
 *  タイマの現在値の読みだし
 *
 *  割込み禁止区間中で呼び出すこと．
 */
Inline CLOCK
hw_timer_get_current(void)
{
    return(TSYS_TC);
}

Inline BOOL
hw_timer_fetch_interrupt(void)
{
    return(TSYS_IR);
}

#endif /* _MACRO_ONLY */
#endif /* _HW_TIMER_H_ */
